// This file is generated by kconfig_compiler from 802-11-wireless-security.kcfg.
// All changes you do to this file will be lost.
#ifndef WIRELESSSECURITYDBUS_H
#define WIRELESSSECURITYDBUS_H

#include <nm-setting-wireless-security.h>

#include <kdebug.h>
#include <kcoreconfigskeleton.h>
#include "settingdbus.h"
#include "nm09dbus_export.h"
namespace Knm{
    class WirelessSecuritySetting;
}

class NM09DBUS_EXPORT WirelessSecurityDbus : public SettingDbus
{
  public:
    WirelessSecurityDbus(Knm::WirelessSecuritySetting * setting, const QString & essid);
    ~WirelessSecurityDbus();
    void fromMap(const QVariantMap&);
    QVariantMap toMap();
    QVariantMap toSecretsMap();
  private:
    QString hashWpaPsk(const QString & plainText);
    QString hashWepPassphrase(const QString & plainText);
    QString m_essid;
};
#endif

