// This file is generated by kconfig_compiler from ipv4.kcfg.
// All changes you do to this file will be lost.

#include "ipv4.h"

using namespace Knm;

Ipv4Setting::Ipv4Setting() : Setting(Setting::Ipv4),
    mMethod(Ipv4Setting::EnumMethod::Automatic),
    mIgnoredhcpdns(false),
    mIgnoreautoroute(false),
    mNeverdefault(false),
    mMayfail(false)
{
}

Ipv4Setting::Ipv4Setting(Ipv4Setting *setting) : Setting(setting)
{
    setMethod(setting->method());
    setDns(setting->dns());
    setDnssearch(setting->dnssearch());
    setAddresses(setting->addresses());
    setIgnoredhcpdns(setting->ignoredhcpdns());
    setIgnoreautoroute(setting->ignoreautoroute());
    setNeverdefault(setting->neverdefault());
    setDhcpclientid(setting->dhcpclientid());
    setDhcphostname(setting->dhcphostname());
    setRoutes(setting->routes());
    setMayfail(setting->mayfail());
}

Ipv4Setting::~Ipv4Setting()
{
}

QString Ipv4Setting::name() const
{
  return QLatin1String("ipv4");
}
bool Ipv4Setting::hasSecrets() const
{
  return false;
}
