// This file is generated by kconfig_compiler from 802-3-ethernet.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_WIREDSETTING_H
#define KNM_WIREDSETTING_H

#include <kglobal.h>
#include <kdebug.h>
#include <kcoreconfigskeleton.h>
#include "setting.h"
#include "knminternals_export.h"
namespace Knm {

class KNMINTERNALS_EXPORT WiredSetting : public Setting
{
  public:
    class EnumPort
    {
      public:
      enum type { tp, aui, bnc, mii, COUNT };
    };
    class EnumDuplex
    {
      public:
      enum type { half, full, COUNT };
    };

    WiredSetting( );
    WiredSetting(WiredSetting *);
    ~WiredSetting();

    QString name() const;

    bool hasSecrets() const;

    /**
      Set Port
    */
    void setPort( int v )
    {
        mPort = v;
    }

    /**
      Get Port
    */
    int port() const
    {
      return mPort;
    }

    /**
      Set Speed
    */
    void setSpeed( uint v )
    {
        mSpeed = v;
    }

    /**
      Get Speed
    */
    uint speed() const
    {
      return mSpeed;
    }

    /**
      Set Duplex
    */
    void setDuplex( int v )
    {
        mDuplex = v;
    }

    /**
      Get Duplex
    */
    int duplex() const
    {
      return mDuplex;
    }

    /**
      Set Auto negotiate
    */
    void setAutonegotiate( bool v )
    {
        mAutonegotiate = v;
    }

    /**
      Get Auto negotiate
    */
    bool autonegotiate() const
    {
      return mAutonegotiate;
    }

    /**
      Set MAC Address
    */
    void setMacaddress( const QByteArray & v )
    {
        mMacaddress = v;
    }

    /**
      Get MAC Address
    */
    QByteArray macaddress() const
    {
      return mMacaddress;
    }

    /**
     *  Set cloned MAC Address
     */
    void setClonedmacaddress( const QByteArray & v )
    {
        mClonedmacaddress = v;
    }

    /**
     *  Get cloned MAC Address
     */
    QByteArray clonedmacaddress()
    {
        return mClonedmacaddress;
    }


    /**
      Set MTU
    */
    void setMtu( uint v )
    {
        mMtu = v;
    }

    /**
      Get MTU
    */
    uint mtu() const
    {
      return mMtu;
    }

  protected:

    // 802-3-ethernet
    int mPort;
    uint mSpeed;
    int mDuplex;
    bool mAutonegotiate;
    QByteArray mMacaddress;
    QByteArray mClonedmacaddress;
    uint mMtu;

  private:
};

}

#endif

