// This file is generated by kconfig_compiler from 802-11-wireless-security.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_WIRELESSSECURITYPERSISTENCE_H
#define KNM_WIRELESSSECURITYPERSISTENCE_H

#include <kdebug.h>
#include "settingpersistence.h"
#include "internals/settings/802-11-wireless-security.h"
namespace Knm {

class WirelessSecuritySetting;

class WirelessSecurityPersistence : public SettingPersistence
{
  public:
    WirelessSecurityPersistence( WirelessSecuritySetting * setting, KSharedConfig::Ptr config, SettingPersistence::SecretStorageMode mode = SettingPersistence::Secure);
    ~WirelessSecurityPersistence();
    void load();
};
}

#endif

