// This file is generated by kconfig_compiler from bluetooth.kcfg.
// All changes you do to this file will be lost.

#include "bluetooth.h"

using namespace Knm;

BluetoothSetting::BluetoothSetting() : Setting(Setting::Bluetooth)
{
}

BluetoothSetting::BluetoothSetting(BluetoothSetting *setting) : Setting(setting)
{
    setNetworkname(setting->networkname());
    setBdaddr(setting->bdaddr());
    setNetworktype(setting->networktype());
}

BluetoothSetting::~BluetoothSetting()
{
}

QString BluetoothSetting::name() const
{
  return QLatin1String("bluetooth");
}
bool BluetoothSetting::hasSecrets() const
{
  return false;
}
