// This file is generated by kconfig_compiler from ipv4.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_IPV4PERSISTENCE_H
#define KNM_IPV4PERSISTENCE_H

#include <kdebug.h>
#include "settingpersistence.h"
#include "internals/settings/ipv4.h"

namespace Knm {

class Ipv4Setting;

class Ipv4Persistence : public SettingPersistence
{
  public:
    Ipv4Persistence( Ipv4Setting * setting, KSharedConfig::Ptr config, SettingPersistence::SecretStorageMode mode = SettingPersistence::Secure);
    ~Ipv4Persistence();
    void load();
};
}

#endif

