/*
Copyright 2008 Will Stephenson <wstephenson@kde.org>
Copyright 2010 Lamarque Souza <lamarque@gmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License or (at your option) version 3 or any later version
accepted by the membership of KDE e.V. (or its successor approved
by the membership of KDE e.V.), which shall act as a proxy
defined in Section 14 of version 3 of the license.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MM05_MODEMCDMAINTERFACE_H
#define MM05_MODEMCDMAINTERFACE_H

#include "modeminterface.h"
#include "solid/control/ifaces/modemcdmainterface.h"
#include "dbus/generic-types.h"

class MMModemCdmaInterfacePrivate;

class KDE_EXPORT MMModemCdmaInterface : public MMModemInterface, virtual public Solid::Control::Ifaces::ModemCdmaInterface
{
Q_OBJECT
Q_DECLARE_PRIVATE(MMModemCdmaInterface)
Q_INTERFACES(Solid::Control::Ifaces::ModemCdmaInterface)

public:
    MMModemCdmaInterface(const QString & path, MMModemManager * manager, QObject * parent);
    ~MMModemCdmaInterface();

    uint getSignalQuality();
    QString getEsn();
    ServingSystemType getServingSystem();
    RegistrationStateResult getRegistrationState();

Q_SIGNALS:
    void signalQualityChanged(uint signalQuality);
    void registrationStateChanged(const Solid::Control::ModemCdmaInterface::RegistrationState cdma_1x_state,
                                  const Solid::Control::ModemCdmaInterface::RegistrationState evdo_state);
};

#endif

