// This file is generated by kconfig_compiler from 802-11-wireless.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_WIRELESSPERSISTENCE_H
#define KNM_WIRELESSPERSISTENCE_H

#include <kdebug.h>
#include "settingpersistence.h"
#include "internals/settings/802-11-wireless.h"
namespace Knm {

class WirelessSetting;

class WirelessPersistence : public SettingPersistence
{
  public:
    WirelessPersistence( WirelessSetting * setting, KSharedConfig::Ptr config, SettingPersistence::SecretStorageMode mode = SettingPersistence::Secure);
    ~WirelessPersistence();
    void load();
};
}

#endif

