// This file is generated by kconfig_compiler from 802-3-ethernet.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_WIREDPERSISTENCE_H
#define KNM_WIREDPERSISTENCE_H

#include <kdebug.h>
#include "settingpersistence.h"
#include "internals/settings/802-3-ethernet.h"

namespace Knm {

class WiredSetting;

class WiredPersistence : public SettingPersistence
{
  public:
    WiredPersistence( WiredSetting * setting, KSharedConfig::Ptr config, SettingPersistence::SecretStorageMode mode = SettingPersistence::Secure);
    ~WiredPersistence();
    void load();
};
}

#endif

