// This file is generated by kconfig_compiler from serial.kcfg.
// All changes you do to this file will be lost.

#include "serial.h"

using namespace Knm;

SerialSetting::SerialSetting() : Setting(Setting::Serial), mBaud(57600), mBits(8), mParity(EnumParity::None), mStopbits(1), mSenddelay(0)
{
}

SerialSetting::SerialSetting(SerialSetting *setting) : Setting(setting)
{
    setBaud(setting->baud());
    setBits(setting->bits());
    setParity(setting->parity());
    setStopbits(setting->stopbits());
    setSenddelay(setting->senddelay());
}

SerialSetting::~SerialSetting()
{
}

QString SerialSetting::name() const
{
  return QLatin1String("serial");
}
bool SerialSetting::hasSecrets() const
{
  return false;
}
