// This file is generated by kconfig_compiler from vpn.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_VPNPERSISTENCE_H
#define KNM_VPNPERSISTENCE_H

#include <kdebug.h>
#include "settingpersistence.h"
#include "internals/settings/vpn.h"

#include "../types.h"

namespace Knm {

class VpnSetting;

class VpnPersistence : public SettingPersistence
{
  public:
    VpnPersistence( VpnSetting * setting, KSharedConfig::Ptr config, SettingPersistence::SecretStorageMode mode = SettingPersistence::Secure);
    ~VpnPersistence();
    void load();
    static QStringMap stringMapFromStringList(const QStringList &);
};
}

#endif

