// This file is generated by kconfig_compiler from bluetooth.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_BLUETOOTH_H
#define KNM_BLUETOOTH_H

#include <kglobal.h>
#include <kconfigskeleton.h>
#include <kdebug.h>
#include "setting.h"
#include "knminternals_export.h"

namespace Knm {

class KNMINTERNALS_EXPORT BluetoothSetting : public Setting
{
  public:

    BluetoothSetting( );
    BluetoothSetting(BluetoothSetting *);
    ~BluetoothSetting();

    QString name() const;

    bool hasSecrets() const;

    /**
      Set Network Name
    */
    void setNetworkname( const QString & v )
    {
        mNetworkname = v;
    }

    /**
      Get Network Name
    */
    QString networkname() const
    {
      return mNetworkname;
    }


    /**
      Set Bdaddr
    */
    void setBdaddr( const QByteArray & v )
    {
        mBdaddr = v;
    }

    /**
      Get Bdaddr
    */
    QByteArray bdaddr() const
    {
      return mBdaddr;
    }


    /**
      Set Network Type
    */
    void setNetworktype( const QString & v )
    {
        mNetworktype = v;
    }

    /**
      Get Network Type
    */
    QString networktype() const
    {
      return mNetworktype;
    }

  protected:

    // bluetooth
    QString mNetworkname;
    QByteArray mBdaddr;
    QString mNetworktype;

  private:
};

}

#endif

