// This file is generated by kconfig_compiler from 802-1x.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_SECURITY8021XPERSISTENCE_H
#define KNM_SECURITY8021XPERSISTENCE_H

#include <kdebug.h>
#include "settingpersistence.h"
#include "internals/settings/802-1x.h"

namespace Knm {

class Security8021xSetting;

class Security8021xPersistence : public SettingPersistence
{
  public:
    Security8021xPersistence( Security8021xSetting * setting, KSharedConfig::Ptr config, SettingPersistence::SecretStorageMode mode = SettingPersistence::Secure);
    ~Security8021xPersistence();
    void load();
};
}

#endif

