// This file is generated by kconfig_compiler from gsm.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_BLUETOOTHPERSISTENCE_H
#define KNM_BLUETOOTHPERSISTENCE_H

#include <kdebug.h>
#include "settingpersistence.h"
#include "internals/settings/bluetooth.h"

namespace Knm {

class BluetoothSetting;

class BluetoothPersistence : public SettingPersistence
{
  public:
    BluetoothPersistence( BluetoothSetting * setting, KSharedConfig::Ptr config, SettingPersistence::SecretStorageMode mode = SettingPersistence::Secure);
    ~BluetoothPersistence();
    void load();
};
}

#endif

