/*
    Copyright (C) 2009 Dario Freddi <drf@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#include "wicdconnectionhandler.h"

#include <connection.h>
#include <interfaceconnection.h>

#include <kdebug.h>

void WicdConnectionHandler::handleRemove(Knm::Activatable* )
{

}

void WicdConnectionHandler::handleUpdate(Knm::Activatable* )
{

}

void WicdConnectionHandler::handleAdd(Knm::Activatable *added)
{
}

void WicdConnectionHandler::handleRemove(Knm::Connection* )
{

}

void WicdConnectionHandler::handleUpdate(Knm::Connection* )
{

}

void WicdConnectionHandler::handleAdd(Knm::Connection* )
{

}

