// This file is generated by kconfig_compiler from 802-11-wireless.kcfg.
// All changes you do to this file will be lost.
#ifndef WIRELESSDBUS_H
#define WIRELESSDBUS_H

#include <nm-setting-wireless.h>

#include <kdebug.h>
#include <kcoreconfigskeleton.h>
#include "settingdbus.h"
#include "nm09dbus_export.h"
namespace Knm{
    class WirelessSetting;
}

class NM09DBUS_EXPORT WirelessDbus : public SettingDbus
{
  public:
    WirelessDbus(Knm::WirelessSetting * setting);
    ~WirelessDbus();
    void fromMap(const QVariantMap&);
    QVariantMap toMap();
    QVariantMap toSecretsMap();
};
#endif

