// This file is generated by kconfig_compiler from ipv6.kcfg.
// All changes you do to this file will be lost.
#ifndef KNM_IPV6PERSISTENCE_H
#define KNM_IPV6PERSISTENCE_H

#include <kdebug.h>
#include <internals/settings/ipv6.h>
#include "settingpersistence.h"

namespace Knm {

class Ipv6Setting;

class Ipv6Persistence : public SettingPersistence
{
  public:
    Ipv6Persistence( Ipv6Setting * setting, KSharedConfig::Ptr config, SettingPersistence::SecretStorageMode mode = SettingPersistence::Secure);
    ~Ipv6Persistence();
    void load();
};
}

#endif

